package meerkat.voting;

import meerkat.crypto.concrete.ECElGamalEncryption;
import meerkat.crypto.concrete.GlobalCryptoSetup;
import meerkat.protobuf.ConcreteCrypto;
import meerkat.protobuf.Voting.*;
import meerkat.crypto.Encryption;
import meerkat.protobuf.Crypto.*;

import java.io.IOException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.util.Random;
import java.util.concurrent.ArrayBlockingQueue;

import com.google.protobuf.ByteString;
import org.bouncycastle.jce.spec.*;
import org.bouncycastle.math.ec.ECPoint;
import org.factcenter.qilin.primitives.concrete.ECElGamal;
import org.factcenter.qilin.primitives.concrete.ECGroup;
import org.factcenter.qilin.primitives.generic.ElGamal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class VotingBoothToy implements VotingBooth, Runnable {

	//private ElectionParams m_electionParams;		
	private EncryptionPublicKey m_ballotEncryptionKey;
	//private List<BallotQuestionNew> l_questions;
	//private BallotQuestionNew a_questions[];
	//private BallotAnswer a_answers[];
	
	private ArrayBlockingQueue<VBMessage> a_queue;
	static private int m_queueSize = 5;

	private VotingBooth.UI m_ui;
	
	//TODO: where do I use these?
	//SignatureVerificationKey a_signatureKeys[];
	
	Encryption m_encryptor;
	Random m_random;

    Logger logger;


    public VotingBoothToy () {
		logger = LoggerFactory.getLogger(VotingBoothToy.class);
		a_queue = new ArrayBlockingQueue<VBMessage> (m_queueSize, true);

	}
	
	public void registerUI (UI ui) {
		m_ui = ui;
	}
	
	public void run () {
		System.err.println("VB controller is up and running");
		
		while (true) {
			VBMessage msg;
			try {
                msg = a_queue.take();
			}
			catch (InterruptedException e) {
				System.err.println("VB controller interrupted!!");
				break;
			}
			if (msg.getType() == VBMessage.VBMessageType.VB_ENCRYPTION_QUERY) {
				handleEncryptionQuery (msg);
			}
			else {
				System.err.println("VB controller unknown type of message!!");
				break;
			}
		}
	}
	
	
	private void handleEncryptionQuery (VBMessage msg) {
		assert msg.getType() == VBMessage.VBMessageType.VB_ENCRYPTION_QUERY;
		PlaintextBallot ptb = msg.getPlaintextBallot();
		
		// encrypt
		RerandomizableEncryptedMessage encMsg;
		EncryptionRandomness rnd = m_encryptor.generateRandomness(m_random);
		try {
			// simulate taking a lot of time
			Thread.sleep(5000);
			encMsg = m_encryptor.encrypt(ptb, rnd);
		}
		catch (InterruptedException e) {
			System.err.println("debug: VotingBoothToy encrypt interrupted!");
			return;
		}
		catch (IOException e) {
			System.err.println("debug: VotingBoothToy encrypt IOException!");
			return;
		}

		EncryptedBallot encBallot = EncryptedBallot.newBuilder()
				                         .setSerialNumber(ptb.getSerialNumber())
				                         .setData(encMsg)
				                         .build();
		RandomnessGenerationProof proof = RandomnessGenerationProof.newBuilder()
				                         .setData(ByteString.copyFrom(new byte[0]))
				                         .build();
		BallotSecrets secrets = BallotSecrets.newBuilder()
				                         .setPlaintextBallot(ptb)
				                         .setEncryptionRandomness(rnd)
				                         .setProof(proof)
				                         .build();

		m_ui.commitToEncryptedBallot(encBallot, secrets);
	}
	
	
	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#init(meerkat.protobuf.Voting.ElectionParams, meerkat.protobuf.Voting.BoothParams)
	 */
	@Override
	public void init(ElectionParams globalParams, BoothParams boothParams) {
		System.err.println ("debug VB: init.");
		this.m_ballotEncryptionKey = globalParams.getBallotEncryptionKey();
		//this.l_questions =  globalParams.getQuestionsList();
		
//		List<SignatureVerificationKey> l_signatureKeys =  boothParams.getPscVerificationKeysList();
//		a_signatureKeys = new SignatureVerificationKey[l_signatureKeys.size()];
//		int i = 0;
//		for (SignatureVerificationKey q: l_signatureKeys) {
//			a_signatureKeys[i] = q;
//			++i;
//		}

        m_random = new Random();
        ECGroup group = new ECGroup("secp256k1");
        java.math.BigInteger sk = ECElGamal.generateSecretKey(group, m_random);
        ElGamal.SK<ECPoint> key = new ECElGamal.SK(group, sk);
        ConcreteCrypto.ElGamalPublicKey serializedPk = serializePk(group, key);
        m_encryptor = new ECElGamalEncryption();
        m_encryptor.init(serializedPk);


        //TODO: generate my own signature key
        System.err.println("Here the controller should create its own signature key");

	}


    /**
     * This method was copied from a test file
     * It must be resolved in another way!
     * //TODO:  where should this method be declared?
     */
    public ConcreteCrypto.ElGamalPublicKey serializePk(ECGroup group, ElGamal.PK<ECPoint> pk) {
        ECPoint pkPoint = pk.getPK();
        ECParameterSpec params = group.getCurveParams();

        ECPublicKeySpec pubKeySpec = new ECPublicKeySpec(pkPoint, params);

        try {
            final String ENCRYPTION_KEY_ALGORITHM = "ECDH";

            KeyFactory fact = KeyFactory.getInstance(ENCRYPTION_KEY_ALGORITHM,
                    GlobalCryptoSetup.getBouncyCastleProvider());
            PublicKey javaPk = fact.generatePublic(pubKeySpec);
            ConcreteCrypto.ElGamalPublicKey serializedPk = ConcreteCrypto.ElGamalPublicKey.newBuilder()
                    .setSubjectPublicKeyInfo(ByteString.copyFrom(javaPk.getEncoded())).build();

            return serializedPk;
        } catch (NoSuchAlgorithmException |InvalidKeySpecException e) {
            System.err.println("Should never happen!");
            throw new RuntimeException("Error converting public key!", e);
        }
    }

    /* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#submitBallot(meerkat.protobuf.Voting.PlaintextBallot)
	 */
	@Override
	public void submitBallot(PlaintextBallot ballot) {
		VBMessage msg = VBMessage.newEncryptionQuery(ballot);
		try {
            a_queue.put(msg);
		}
		catch (InterruptedException e) {
			System.err.println("SubmitBallot interrupted!!");
		}
	}

	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#cancelBallot()
	 */
	/*@Override
	public void cancelBallot() {
		// TODO Auto-generated method stub

	}*/

	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#voterCastOrAudit(boolean)
	 */
	/*@Override
	public void voterCastOrAudit(boolean castVote) {
		// TODO Auto-generated method stub

	}*/

}



/*

//A ballot question. This is an opaque
//data type that is parsed by the UI to display
//the question.
message BallotQuestion {
 bytes data = 1;
}

//An answer to a specific ballot question.
//The answer is a vector of signed integers,
//to encompass voting schemes such as ranked voting
//and STV.
message BallotAnswer {
 repeated sint64 answer = 1 [packed=true];
}

message EncryptedBallot {
    uint64 serialNumber = 1; // Ballot serial number

    RerandomizableEncryptedMessage data = 2;
}

message BallotSecrets {
    PlaintextBallot plaintext_ballot = 1;

    EncryptionRandomness encryption_randomness = 2;
    RandomnessGenerationProof proof = 3;
}

message BoothParams {
 repeated SignatureVerificationKey pscVerificationKeys = 1;

}

//A table to translate to and from compactly encoded answers
//and their human-understandable counterparts.
//This should be parsable by the UI
message BallotAnswerTranslationTable {
 bytes data = 1;
}

message ElectionParams {
 // TODO: different sets of keys for different roles?
 //repeated SignatureVerificationKey trusteeVerificationKeys = 1;

 // How many trustees must participate in a signature for it to be considered valid.
 //uint32 trusteeSignatureThreshold = 2;

 // The key used to encrypt ballots. The corresponding private key
 // is shared between the trustees.
 EncryptionPublicKey ballotEncryptionKey = 3;

 // Verification keys for valid mixers.
 //repeated SignatureVerificationKey mixerVerificationKeys = 4;

 // How many mixers must participate for the mixing to be considered valid
 //uint32 mixerThreshold = 5;

 // Candidate list (or other question format)
 repeated BallotQuestion questions = 6;

 // Translation table between answers and plaintext encoding
 BallotAnswerTranslationTable answerTranslationTable = 7;

}
*/