package meerkat.voting;

import com.google.protobuf.ByteString;

import meerkat.protobuf.Crypto.SignatureType;
import meerkat.protobuf.Crypto.SignatureVerificationKey;
import meerkat.protobuf.Voting.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class VotingBoothToyDemoRun {
	final static Logger logger = LoggerFactory.getLogger(VotingBoothToyDemoRun.class);

	static int N_SIGNATURE_VERIFICATION_KEYS = 3;
	
	public static void main (String[] args)
	{
		VotingBoothToy vbController = new VotingBoothToy ();
		VotingBoothToyConsoleUI ui = new VotingBoothToyConsoleUI();

        logger.debug("Running with {} args", args.length);
		vbController.registerUI (ui);
		ui.registerVBController(vbController);


		
		BoothParams boothParams = generateDemoBoothParams();
		ElectionParams electionParams = generateDemoElectionParams();
		
		vbController.init(electionParams, boothParams);
		ui.init(electionParams);
		
		Thread controllerThread = new Thread(vbController);
		Thread uiThread = new Thread(ui);

		controllerThread.start();
		uiThread.start();	
	}

	
	public static BoothParams generateDemoBoothParams () {
		BoothParams.Builder bpb = BoothParams.newBuilder();
		SignatureType signatureType = SignatureType.ECDSA;
		for (int i = 0; i < N_SIGNATURE_VERIFICATION_KEYS; ++i) {
			SignatureVerificationKey verifiationKey = SignatureVerificationKey.newBuilder()
														.setType(signatureType)
														.setData(ByteString.copyFrom(new byte[0]))
														.build();
			bpb.addPscVerificationKeys(i, verifiationKey);
		}
		return bpb.build();

	}

	public static BallotQuestionNew generateBallotQuestion(String questionStr, String descriptionStr, String[] answers) {
		UIElement question = UIElement.newBuilder()
		              .setType(UIElementDataType.TEXT)
		              .setData(stringToBytes(questionStr))
		              .build();
		
		UIElement description = UIElement.newBuilder()
				      .setType(UIElementDataType.TEXT)
		              .setData(stringToBytes(descriptionStr))
		              .build();
		
		BallotQuestionNew.Builder bqb = BallotQuestionNew.newBuilder();
		bqb.setIsMandatory(false);
		bqb.setQuestion(question);
		bqb.setDescription(description);
		for (String answerStr : answers) {
			UIElement answer = UIElement.newBuilder()
					                    .setType(UIElementDataType.TEXT)
					                    .setData(stringToBytes(answerStr))
					                    .build();
			bqb.addAnswer(answer);
		}
		
		return bqb.build();
	}
	
	
	
	public static ByteString stringToBytes (String s) {
		return ByteString.copyFromUtf8(s);
	}
	
	public static ElectionParams generateDemoElectionParams () {
		String[] answers1 = {"Blue", "Red", "Green", "Purple"};
		BallotQuestionNew question1 = generateBallotQuestion("What is your favorite color?", "Pick one answer", answers1);

		String[] answers2 = {"Miranda Kerr", "Doutzen Kroes", "Moran Atias", "Roslana Rodina", "Adriana Lima"};
		BallotQuestionNew question2 = generateBallotQuestion("Which model do you like", "Mark as many as you want", answers2);

		String[] answers3 = {"Clint Eastwood", "Ninja", "Sonic", "Tai-chi", "Diablo"};
		BallotQuestionNew question3 = generateBallotQuestion("Good name for a cat", "Pick the best one", answers3);

		ElectionParams.Builder epb = ElectionParams.newBuilder();
		epb.setTrusteeSignatureThreshold(5);
		epb.addQuestion(question1);
		epb.addQuestion(question2);
		epb.addQuestion(question3);
		
		return epb.build();
	}

}
